#!/usr/bin/perl -w
# script to query the Varnish manager process and collect statistics
# for Cacti. Based on the python script by dmuntean.
#

use strict;
use Net::Telnet ();

my $host=shift || 'localhost';		# default host
my $port=shift || 5999;			# default port number

my $t=new Net::Telnet( Timeout => 10,
			Host=>$host,
			Port=>$port,
			Prompt=>'//'
		);
$t->open() || die "Can't open connection!" ;
$t->put(String=>"stats\n");
my $l=$t->get();
$t->close();

# process data
my @values=split /\n/,$l;
foreach my $x (@values) {

	# a little bit of processing is required, the varnishstat
	# command returns values in a line, with the name of the value,
	# the value, a second value (not really sure what is it) and
	# the description.

	$x=~s/^([\w]+)[\s]+([\w]+)[\s]+([\w.]+)[\s]+(.*)$/$1:$2:$3:$4/;
	my ($id, $val, $v2, $desc)=split /:/,$x;

	if( $desc ) {
		$desc=~s/ /_/g;
		$desc=~s/[()]//g;
		$desc=lc $desc;

		# the bytes_allocated and bytes_free are, unfortunately,
		# not unique, so I replace them with 'cache_allocated'
		# and 'cache_free'.
		if( $desc =~ /^bytes_allocated$/ ) {
			$desc='cache_allocated';
		}
		if( $desc=~ /^bytes_free$/ ) {
			$desc='cache_free';
		}
	
		# cache_hits_for_pass = uncachables objects
		if( $desc=~ /^cache_hits_for_pass$/ ) {
			$desc='uncachables';
		}
		print $desc.":".$val." ";
	}
}

print "\n";

